# command.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import stdhome.the as the
from stdhome.walker.walker import Walker


class Command:
	"""Base class for command classes.
	"""

	@staticmethod
	def expand_files( files, recurse = True ):
		"""Returns a unique, sorted list of relative files, calculated from the list
		provided, which is made up from individual files and directories
		relative to the CWD (and which must be contained within the home
		directory, although the files need not actually exist in the home
		directory).  All files must exist in the repository.  Directories are
		recursed in to as required.

		"""

		ret = set()
		home_dir_prefix = os.path.realpath( the.full_home_dir ) + os.sep

		# iterate through file list
		for file in files:
			parts = os.path.split( file )
			abs_file = os.path.join(
				os.path.realpath( parts[ 0 ] ), parts[ 1 ] )

			# check the file is in the home directory
			if abs_file[ : len( home_dir_prefix ) ] != home_dir_prefix:
				raise the.program.FatalError(
					'not under home directory: %s' % abs_file )

			# relative file
			rel_file = abs_file[ len( home_dir_prefix ) : ]

			# check if file exists in repository
			repo_file = os.path.join( the.repo.full_dir, rel_file )
			if not os.path.lexists( repo_file ):
				raise the.program.FatalError(
					'not managed by stdhome: %s' % rel_file )

			# append the file or directory tree
			ret.update( Walker.generate_walk_list(
				the.repo.full_dir, rel_file, recurse ) )

		return sorted( set ( ret ) )
